import '../../agent/action/null_actions/null_root_action_impl.dart';
import './interface/action.dart';

import 'package:flutter/widgets.dart';

/// Class which handles the action count so it is not possible
/// to create an action with the same id.
class ActionCounter {
  static ActionCounter? _instance;

  int _count;

  /// Contains the current open actions
  Map<int, DynatraceAction> currentActions = {};

  /// Private constructor to initialize the counter
  ActionCounter.private(this._count);

  /// Initializing the counter and set the counter to zero.
  factory ActionCounter() {
    _instance ??= ActionCounter.private(0);
    return _instance!;
  }

  @visibleForTesting

  /// Resetting the count to zero.
  void resetCount() {
    _count = 0;
  }

  /// Get the current count.
  int getCount() {
    return _count;
  }

  /// Returns the id of the action created last
  DynatraceAction getCurrentAction() {
    if (currentActions.containsKey(_count - 1)) {
      return currentActions[_count - 1]!;
    }
    return DynatraceNullRootAction();
  }

  /// Set a new [action] with a [key]. This action will be queued
  /// into the open actions map.
  void setOpenAction(int key, DynatraceAction action) {
    currentActions[_count] = action;
    _count++;
  }

  /// Remove a action from the map with the [key]
  void removeClosedAction(int key) {
    if (currentActions.containsKey(key)) {
      currentActions.remove(key);
    }
  }
}
