import '../../agent/interface/web_request_timing.dart';
import '../util/logger.dart';
import '../util/string_utils.dart';

import 'package:flutter/services.dart';

/// Class which is able to be used for tracking a web request.
class DynatraceWebRequestTiming extends WebRequestTiming {
  final MethodChannel _methodChannel;
  final String _requestTag;
  final String _url;

  /// Constructor for initializing a web request tracking
  DynatraceWebRequestTiming(this._requestTag, this._url, this._methodChannel);

  // Starts the timing of a web request on a user action.
  @override
  void startWebRequestTiming() {
    if (_requestTag != "" && !StringUtils.isStringNullEmptyOrWhitespace(_url)) {
      Logger()
          .d("WebRequestTiming startWebRequestTiming()", logType: LogType.Info);
      _methodChannel.invokeMethod("startWebRequestTiming",
          <String, dynamic>{"requestTag": _requestTag, "url": _url});
    } else {
      Logger().d(
          "WebRequestTiming startWebRequestTiming(): WebRequestTiming could not be created!",
          logType: LogType.Error);
    }
  }

  // Stops the timing of a web request on a user action.
  @override
  void stopWebRequestTiming(int responseCode, String? responseMessage) {
    if (_requestTag != "" && !StringUtils.isStringNullEmptyOrWhitespace(_url)) {
      Logger().d(
          "WebRequestTiming stopWebRequestTiming($responseCode, $responseMessage)",
          logType: LogType.Info);
      _methodChannel.invokeMethod("stopWebRequestTiming", <String, dynamic>{
        "requestTag": _requestTag,
        "url": _url,
        "responseCode": responseCode,
        "responseMessage": responseMessage
      });
    } else {
      Logger().d(
          "WebRequestTiming stopWebRequestTiming(responseCode, responseMessage): WebRequestTiming could not be stopped! \n_requestTag: $_requestTag \n_url: $_url \nresponseCode: $responseCode \nresponseMessage: $responseMessage",
          logType: LogType.Error);
    }
  }

  @override
  String getRequestTag() {
    Logger().d("WebRequestTiming getRequestTag()", logType: LogType.Info);
    return _requestTag;
  }

  @override
  String getRequestTagHeader() {
    Logger().d("WebRequestTiming getRequestTagHeader()", logType: LogType.Info);
    return "x-dynatrace";
  }
}
