import '../../agent/agent_impl.dart';
import '../../agent/interface/web_request_timing.dart';

import 'package:http/http.dart';

import '../util/logger.dart';

class DynatraceHttpClient extends BaseClient {
  final DynatraceImpl _dynatrace;
  final Client _client;

  /// Constructor which wraps a [client]
  DynatraceHttpClient(this._dynatrace, {Client? client})
      : _client = client ?? Client();

  @override
  Future<StreamedResponse> send(BaseRequest request) async {
    StreamedResponse? response;
    String? header =
        await _dynatrace.getHTTPTagForWebRequest(request.url.toString());
    if (header != null && header.isNotEmpty) {
      request.headers['x-dynatrace'] = header;
    }
    if (request.headers.containsKey('x-dynatrace')) {
      WebRequestTiming timing = _dynatrace.createWebRequestTiming(
          request.headers['x-dynatrace']!, request.url.toString());
      try {
        timing.startWebRequestTiming();
        response = await _client.send(request);
        timing.stopWebRequestTiming(response.statusCode, response.reasonPhrase);
      } catch (error) {
        timing.stopWebRequestTiming(-1, error.toString());
        rethrow;
      }
      return response;
    } else {
      Logger().d("The x-dynatrace header could not be set on the request!",
          logType: LogType.Info);
      return await _client.send(request);
    }
  }

  @override
  void close() => _client.close();
}
