import '../../../cli/util/pathConstants.dart';

/// Command Line arguments that can be passed via our CLI command
class CommandLineArguments {
  final bool? _debug;
  final bool? _uninstall;
  final String? _configPath;
  final String? _gradlePath;
  final String? _plistPath;

  CommandLineArguments._private(this._debug, this._uninstall, this._configPath,
      this._gradlePath, this._plistPath);

  /// Parses the arguments that are passed in via CLI
  static CommandLineArguments parseArgumentList(
      PathConstants paths, List<String> arguments) {
    return CommandLineArguments._private(
        _readValueFromArguments(
            arguments, "--debug", false, (argument) => true),
        _readValueFromArguments(
            arguments, "--uninstall", false, (argument) => true),
        _readValueFromArguments(arguments, "--config",
            paths.getConfigurationPath(), (argument) => argument.split("=")[1]),
        _readValueFromArguments(arguments, "--gradle", paths.getAndroidGradle(),
            (argument) => argument.split("=")[1]),
        _readValueFromArguments(arguments, "--plist", paths.getIosPListFile(),
            (argument) => argument.split("=")[1]));
  }

  static dynamic _readValueFromArguments(List<String> arguments, String key,
      dynamic defaultValue, Function result) {
    for (int i = 0; i < arguments.length; i++) {
      if (arguments[i].startsWith(key)) {
        return result(arguments[i]);
      }
    }

    return defaultValue;
  }

  /// Argument for the path of the iOS plist file
  String? get plistPath => _plistPath;

  /// Argument for the path of the .gradle file
  String? get gradlePath => _gradlePath;

  /// Argument for the path of the dynatrace.config.yaml
  String? get configPath => _configPath;

  /// Arguments containing the debug flag
  bool? get isDebug => _debug;

  /// Arguments containing the uninstall flag
  bool? get isUninstall => _uninstall;
}
