import '../../../cli/config/model/ios_config.dart';
import '../../../cli/config/model/android_config.dart';

import 'package:yaml/yaml.dart';

/// Configuration for the native Android and iOS agent
/// which will be loaded from the .yaml config file
class Configuration {
  AndroidConfiguration? _androidConfiguration;
  IosConfiguration? _iosConfiguration;

  /// Constructor which accepts the [contentYaml] of the
  /// dynatrace.config.yaml file
  Configuration(String contentYaml) {
    YamlMap? mapYaml = loadYaml(contentYaml);

    if (mapYaml != null && mapYaml["ios"] != null) {
      _iosConfiguration = IosConfiguration(mapYaml["ios"]["config"]);
    } else {
      _iosConfiguration = IosConfiguration(null);
    }

    if (mapYaml != null && mapYaml["android"] != null) {
      YamlMap androidMap = mapYaml.nodes["android"] as YamlMap;
      if (androidMap.nodes["config"] != null) {
        _androidConfiguration = AndroidConfiguration(
            androidMap.nodes["config"]!.span.text.replaceAll("\"", ""));
      } else {
        _androidConfiguration = AndroidConfiguration(null);
      }
    } else {
      _androidConfiguration = AndroidConfiguration(null);
    }
  }

  /// Returning the configuration for the Android agent
  AndroidConfiguration? getAndroidConfiguration() {
    return _androidConfiguration;
  }

  /// Returning the configuration for the iOS Agent
  IosConfiguration? getIosConfiguration() {
    return _iosConfiguration;
  }
}
