// ignore_for_file: file_names

import 'package:path/path.dart' as p;
import 'dart:io';

/// Class which contains all the paths that are used within the plugin
class PathConstants {
  /// Path of the Flutter application
  String getApplicationPath() {
    return p.current;
  }

  /// Path of the Android folder within the Flutter application
  String getAndroidFolder() {
    return p.join(getApplicationPath(), "android");
  }

  /// Path to the build.gradle file of the Android Flutter project
  /// within the application
  String getAndroidGradle() {
    return p.join(getAndroidFolder(), "build.gradle");
  }

  /// Path to the internal dynatrace.gradle file which contains the
  /// configuration for the Android agent
  String getDynatraceGradle(Directory gradleDir) {
    return p.join(gradleDir.path, "dynatrace.gradle");
  }

  /// Path to the internal plugin.gradle file which contains the
  /// dependency to the Dynatrace gradle plugin
  String getPluginGradle(Directory gradleDir) {
    return p.join(gradleDir.path, "plugin.gradle");
  }

  /// Path of the iOS folder within the Flutter application
  String getIosFolder() {
    return p.join(getApplicationPath(), "ios");
  }

  /// Path to the Info.plist file of the Flutter iOS application
  String getIosPListFile() {
    return p.join(getIosFolder(), "Runner", "Info.plist");
  }

  /// Path to the Dynatrace.plist file used for Flutter iOS Application
  String getIosDynatracePListFile() {
    return p.join(getIosFolder(), "Runner", "Dynatrace.plist");
  }

  /// Path to the configuration file for the Dynatrace Flutter plugin
  String getConfigurationPath() {
    return p.join(getApplicationPath(), "dynatrace.config.yaml");
  }
}
